//
// Copyright (c) 2002-2022 WEINZIERL ENGINEERING GmbH
// All rights reserved.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY DAMAGES OR OTHER LIABILITY,
// WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE
//

#include <stdio.h>
#include <stdlib.h>
#include <kdrive_express.h>

/*******************************
** Private Functions
********************************/

static error_t set_license();

/*******************************
** Main
********************************/

int main(int argc, char* argv[])
{
	uint16_t address = 0x901; /*!< The KNX Group Address (destination address) we use to send with */
	uint8_t value = 0; /*!< The value we send on the bus with the Group Value Write command */
	int32_t ap = 0; /*! the Access Port descriptor */

	/*
		Set here if you want use a secure or unsecured port
		When use_secure is true then set here also the used backbone key.
	*/
	bool_t use_secure = 0; /*! use secure KNXnet/IP routing port (0 = unsecured, 1 = secure) */
	const char backbone_key[] = "00000000000000000000000000000000";

	/* Configure the logging level and console logger */
	kdrive_logger_set_level(KDRIVE_LOGGER_INFORMATION);
	kdrive_logger_console();

	/* Sets the license key */
	if (set_license() != KDRIVE_ERROR_NONE)
	{
		kdrive_logger(KDRIVE_LOGGER_FATAL, "The license key is invalid. Commercial features are disabled");
		return -1;
	}

	/*
		We create a Access Port descriptor. This descriptor is then used for
		all calls to that specific access port.
	*/
	ap = kdrive_ap_create();

	/*
		We check that we were able to allocate a new descriptor
		This should always happen, unless a bad_alloc exception is internally thrown
		which means the memory couldn't be allocated.
	*/
	if (ap == KDRIVE_INVALID_DESCRIPTOR)
	{
		kdrive_logger(KDRIVE_LOGGER_FATAL, "Unable to create access port. This is a terminal failure");
		return -1;
	}

	if (use_secure)
	{
		if (kdrive_ap_set_ip_rout_backbone_key(ap, backbone_key) != KDRIVE_ERROR_NONE)
		{
			kdrive_logger(KDRIVE_LOGGER_FATAL, "Unable to set the backbone key");
		}
	}

	// Use default multicast address and network interface
	if (kdrive_ap_open_ip_rout(ap, 0, 0, use_secure) == KDRIVE_ERROR_NONE)
	{
		if (kdrive_ap_set_ind_addr(ap, 0xAFFE) != KDRIVE_ERROR_NONE)
		{
			kdrive_logger(KDRIVE_LOGGER_FATAL, "Unable to set the individual address");
		}

		/*
			Connect the Packet Trace logging mechanism
			to see the Rx and Tx packets
		*/
		kdrive_ap_packet_trace_connect(ap);

		/* send a 1-Bit boolean GroupValueWrite telegram: on */
		value = 1;
		kdrive_ap_group_write(ap, address, &value, 1);

		/* send a 1-Bit boolean GroupValueWrite telegram: off */
		value = 0;
		kdrive_ap_group_write(ap, address, &value, 1);

		/* close the access port */
		kdrive_ap_close(ap);
	}

	/* releases the access port */
	kdrive_ap_release(ap);

	return 0;
}

/*******************************
** Private Functions
********************************/

/*
	Replace the following licence with your license key
*/
error_t set_license()
{
	char license[] =
	    "[Check]\n"
	    "Key=I1EkM2JvaE1HMEZtIlF5VnJuJn08KHRfV0AwTlEwRyk4NT55JFNvXnU6ZnpzMDI6QV42JWZ4"
	    "fHNed11aOld9WHpGXSEibnkrSDVsZU9WIkljeSQ3NWY7eT4rdEJ6XjhpNnw+OkNEIydTYWNF"
	    "RE15JVdQOno3XmtOJjZaRExwLSV8ZXxOYzlJITlhfFFdOHIjUW43SEshfSx8RWgzSipUZU5X"
	    "ZiJpSnIhRS9pJyQrOis/VVAoSHMkaV5kYGFfaFxhJj5peUtdfVBgMGh7K25bITF7OnhdeTQv"
	    "NiZlWCt4Nls3S31nLUU2LE1OXmhubGtxY2wqJHMxVDd0Tlg4OGlIUCE=\n"
	    "\n"
	    "[Customer]\n"
	    "City=84508 Burgkirchen / Alz\n"
	    "Company=WEINZIERL ENGINEERING GmbH\n"
	    "Country=Germany\n"
	    "Street=Achatz 3\n"
	    "\n"
	    "[LicenseData]\n"
	    "LicNum=kdrive-2012-05-14\n"
	    "LicVersion=71949c2a-be30-4fbe-9c96-d0a617ac6ab8\n"
	    "\n"
	    "[Limitations]"
	    "DateLimit=2012-12-31"
	    "\n"
	    "[ProgramFeatures]\n"
	    "kdriveAccess=0\n"
	    "kdriveServices=0\n";

	return kdrive_lic_set_license(license);
}

