//
// Copyright (c) 2002-2022 WEINZIERL ENGINEERING GmbH
// All rights reserved.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY DAMAGES OR OTHER LIABILITY,
// WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE
//

#include <stdio.h>
#include <stdlib.h>
#include <kdrive_express.h>

#define ERROR_MESSAGE_LEN	(128)	/*!< kdriveExpress Error Messages */
#define MAX_ITEMS (50) /*!< The maximum number of items (struct with tunneling devices) we expect to read at one time */

/*******************************
** Private Functions
********************************/

static error_t set_license();
static void enumerate_routing_devices(int32_t ap);

/*!
	Called when an error occurs
*/
static void error_callback(error_t e, void* user_data);

/*******************************
** Main
********************************/

int main(int argc, char* argv[])
{
	int32_t ap = 0;

	/*
		Configure the logging level and console logger
	*/
	kdrive_logger_set_level(KDRIVE_LOGGER_INFORMATION);
	kdrive_logger_console();

	/*
		We register an error callback as a convenience logger function to
		print out the error message when an error occurs.
	*/
	kdrive_register_error_callback(&error_callback, NULL);

	/*
		Sets the license key
	*/
	if (set_license() != KDRIVE_ERROR_NONE)
	{
		kdrive_logger(KDRIVE_LOGGER_FATAL, "The license key is invalid. Commercial features are disabled");
		return -1;
	}

	/*
		We create a Access Port descriptor. This descriptor is then used for
		all calls to that specific access port.
	*/
	ap = kdrive_ap_create();

	/*
		We check that we were able to allocate a new descriptor
		This should always happen, unless a bad_alloc exception is internally thrown
		which means the memory couldn't be allocated.
	*/
	if (ap == KDRIVE_INVALID_DESCRIPTOR)
	{
		printf("Unable to create access port. This is a terminal failure\n");
		return -1;
	}

	/* enumerate the KNXnet/IP routing devices */
	enumerate_routing_devices(ap);

	/* releases the access port */
	kdrive_ap_release(ap);

	return 0;
}

/*******************************
** Private Functions
********************************/

/*
	Replace the following licence with your license key
*/
error_t set_license()
{
	char license[] =
	    "[Check]\n"
	    "Key=I1EkM2JvaE1HMEZtIlF5VnJuJn08KHRfV0AwTlEwRyk4NT55JFNvXnU6ZnpzMDI6QV42JWZ4"
	    "fHNed11aOld9WHpGXSEibnkrSDVsZU9WIkljeSQ3NWY7eT4rdEJ6XjhpNnw+OkNEIydTYWNF"
	    "RE15JVdQOno3XmtOJjZaRExwLSV8ZXxOYzlJITlhfFFdOHIjUW43SEshfSx8RWgzSipUZU5X"
	    "ZiJpSnIhRS9pJyQrOis/VVAoSHMkaV5kYGFfaFxhJj5peUtdfVBgMGh7K25bITF7OnhdeTQv"
	    "NiZlWCt4Nls3S31nLUU2LE1OXmhubGtxY2wqJHMxVDd0Tlg4OGlIUCE=\n"
	    "\n"
	    "[Customer]\n"
	    "City=84508 Burgkirchen / Alz\n"
	    "Company=WEINZIERL ENGINEERING GmbH\n"
	    "Country=Germany\n"
	    "Street=Achatz 3\n"
	    "\n"
	    "[LicenseData]\n"
	    "LicNum=kdrive-2012-05-14\n"
	    "LicVersion=71949c2a-be30-4fbe-9c96-d0a617ac6ab8\n"
	    "\n"
	    "[Limitations]"
	    "DateLimit=2012-12-31"
	    "\n"
	    "[ProgramFeatures]\n"
	    "kdriveAccess=0\n"
	    "kdriveServices=0\n";

	return kdrive_lic_set_license(license);
}

/*!
	Enumerats the ip routing interfaces
*/
void enumerate_routing_devices(int32_t ap)
{
	ip_rout_info_t items[MAX_ITEMS];
	uint32_t items_length = MAX_ITEMS;
	uint32_t index = 0;

	kdrive_logger(KDRIVE_LOGGER_INFORMATION, "Enumerating KNXnet/IP Routing devices");
	kdrive_logger(KDRIVE_LOGGER_INFORMATION, "=====================================");
	kdrive_logger(KDRIVE_LOGGER_INFORMATION, "");

	if (kdrive_ap_enum_ip_rout(ap, items, &items_length) == KDRIVE_ERROR_NONE)
	{
		kdrive_logger_ex(KDRIVE_LOGGER_INFORMATION, "Found %d used multicast addresse(s)", items_length);

		for (index = 0; index < items_length; ++index)
		{
			kdrive_logger(KDRIVE_LOGGER_INFORMATION, "");
			kdrive_logger_ex(KDRIVE_LOGGER_INFORMATION, "%s on %s", items[index].multicast_address, items[index].iface_address);
			kdrive_logger_ex(KDRIVE_LOGGER_INFORMATION, "Found %u devices", items[index].number_of_devices);
		}

		kdrive_logger(KDRIVE_LOGGER_INFORMATION, "");
	}
}

/*!
	Called when a kdrive error exception is raised.
	The handling in the error callback is typically
	application specific. And here we simply show
	the error message.
*/
void error_callback(error_t e, void* user_data)
{
	if (e != KDRIVE_TIMEOUT_ERROR)
	{
		static char error_message[ERROR_MESSAGE_LEN];
		kdrive_get_error_message(e, error_message, ERROR_MESSAGE_LEN);
		kdrive_logger_ex(KDRIVE_LOGGER_ERROR, "kdrive error: %s", error_message);
	}
}
